<?php
/*
Plugin Name: DD Attachments
Version: 1.0
Description: A plug-in to add and edit attachments
Author: Wouter Dijkstra
Author URI: http://wouterdijkstra.com
*/


/*  Copyright 2018  WOUTER DIJKSTRA  (email : info@wouterdijkstra.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


class dd_attachments {

    private  $post_id;
    private  $post_image_attachments;
    private  $post_other_attachments;
    private  $uploadLink;
    private  $featured_img_url;
    private  $featured_img_id;

	public function __construct() {

		add_action( 'admin_init', array( $this, 'dd_add_style_and_js_dd_attachments' ) );
		add_action( 'admin_init', array( $this, 'set_metabox_remove_default_featured' ) );
		add_filter( "manage_upload_columns", array( $this, 'upload_columns' ) );
		add_action( "manage_media_custom_column", array( $this, 'media_custom_columns' ), 0, 2 );

		add_action( 'wp_ajax_update_media_attach_state', array( $this, 'update_media_attach_state_callback' ) );
		add_action( 'wp_ajax_dd_set_featured_image', array( $this, 'dd_set_featured_image_callback' ) );
		add_action( 'wp_ajax_dd_remove_featured_image', array( $this, 'dd_remove_featured_image_callback' ) );
		add_action( 'wp_ajax_dd_set_attach_order', array( $this, 'dd_set_attach_order' ) );

	}

	public function set_metabox_remove_default_featured() {
		if ( current_user_can( 'upload_files' ) ) {
			add_action( 'add_meta_boxes', array( $this, 'add_dd_attachments_box' ) );
			add_action( 'do_meta_boxes', array( $this, 'dd_attachment_remove_meta_boxes' ) );
			add_action( 'default_hidden_meta_boxes', array( $this, 'hidden_meta_boxes'), 10, 2 );
		}
	}

	public function add_dd_attachments_box() {
		add_meta_box(
			'dd_attachments',
			__( 'Featured Image & Attachments', 'wp' ),
			array( $this,'dd_attachment_custom_box'),
			'',
			'advanced',
			'high'
		);
	}

    /**
     * @param $hidden
     * @param $screen
     * @return array
     */
	function hidden_meta_boxes( $hidden, $screen ) {
        if($screen->post_type === 'attachment') {
            array_push( $hidden,'dd_attachments' );
        }
        return $hidden;
	}

    /**
     *
     */
	public function dd_attachment_custom_box() {
        global $post;
        $parent_post_id = isset($_POST['the_parent_id']) ? intval(esc_sql($_POST['the_parent_id'])) : '';
        $this->post_id = $parent_post_id ? $parent_post_id : $post->ID;
        $this->post_image_attachments = $this->get_post_image_attachments($this->post_id);
        $this->post_other_attachments = $this->get_post_other_attachments($this->post_id);
        $this->uploadLink = esc_url( get_upload_iframe_src( 'image', $this->post_id ) ); // Get WordPress' media upload URL
        $this->featured_img_url = get_the_post_thumbnail_url($this->post_id, 'thumbnail');
        $this->featured_img_id = get_post_thumbnail_id($this->post_id);
        include('dd_templates/metabox-attachments.php');

	}

	public function dd_add_style_and_js_dd_attachments() {
		wp_register_style( 'dd_attachments_styles', plugins_url( '/css/style.css', __FILE__ ) );
		wp_enqueue_style( 'dd_attachments_styles' );
		wp_enqueue_script( 'dd_js_attachments', plugins_url( '/js/default.js', __FILE__ ), array('jquery','jquery-ui-sortable','jquery-ui-droppable'), '' );
	}

	public function upload_columns( $columns ) {
		unset( $columns['parent'] );
		$columns['better_parent'] = "Attached to";

		return $columns;
	}

    public function media_custom_columns( $column_name, $id ) {
        $post = get_post( $id );
        $title = get_post( $post->post_parent ) ? _draft_or_post_title( $post->post_parent ) : '';

        if ( $column_name != 'better_parent' ) : return; endif;

        if ( $post->post_parent > 0 ) : ?>
            <strong><a href="<?php echo get_edit_post_link( $post->post_parent ); ?>"><?php echo $title ?></a></strong>
            <br/>
            <a class="hide-if-no-js" onclick="findPosts.open('media[]','<?php echo $post->ID ?>');return false;" href="#the-list"><?php _e( 'Re-Attach' ); ?></a>

        <?php else : ?>
            <?php _e( '(Unattached)' ); ?><br/>
            <a class="hide-if-no-js" onclick="findPosts.open('media[]','<?php echo $post->ID ?>');return false;" href="#the-list"><?php _e( 'Attach' ); ?></a>
            <?php
        endif;
    }

	public function dd_attachment_remove_meta_boxes() {
		remove_meta_box( 'postimagediv', '', 'side' );
	}

    function dd_set_featured_image_callback() {
        $the_parent_id = intval(esc_sql($_POST['the_parent_id']));
        $image_id = intval(esc_sql($_POST['image_id']));

        set_post_thumbnail( $the_parent_id, $image_id );
        echo get_the_post_thumbnail_url($the_parent_id, 'thumbnail');

        die();
    }

	function dd_set_attach_order() {

        $the_parent_id = intval(esc_sql($_POST['the_parent_id']));
        $attach_order = array();

	    foreach ($_POST['attach_order'] as $rank) {
            $rank = intval(esc_sql($rank));
            array_push($attach_order,$rank);
        }

		update_post_meta( $the_parent_id, 'attach_order', $attach_order );
		die();
	}

	function dd_remove_featured_image_callback() {
		delete_post_thumbnail(intval(esc_sql( $_POST['the_parent_id'] )));
		die();
	}

    function get_post_image_attachments( $post_id ) {

        $args = array(
            'post_type'      => 'attachment',
            'post_mime_type' => 'image',
            'numberposts'    => - 1,
            'post_status'    => null,
            'post_parent'    => $post_id,
            'order'          => 'ASC'
        );

        $attachments     = get_posts( $args );
        $attach_order    = get_post_meta( $post_id, 'attach_order', true );
        $attach_order = $attach_order ? $attach_order : array();
        $listed_attach   = array();
        $unlisted_attach = array();

        foreach ( $attachments as $attachment ) {
            if ( in_array( $attachment->ID, $attach_order ) ) :
                array_push($listed_attach, $attachment);
            else:
                array_push($unlisted_attach, $attachment);
            endif;
        }

        usort( $listed_attach, function ( $a, $b ) use ( $attach_order ) {
            $order    = $attach_order;
            $flipped  = array_flip( $order );
            $leftPos  = $flipped[ $a->ID ];
            $rightPos = $flipped[ $b->ID ];

            return $leftPos >= $rightPos;
        } );

        $attachments = array_merge( $listed_attach, $unlisted_attach );

        return $attachments;
    }

    function get_post_other_attachments($post_id) {
        $unsupported_mimes  = array( 'image/jpeg', 'image/gif', 'image/png', 'image/bmp', 'image/tiff', 'image/x-icon' );
        $all_mimes          = get_allowed_mime_types();
        $accepted_mimes     = array_diff( $all_mimes, $unsupported_mimes );

        $args = array(
            'post_type'      => 'attachment',
            'post_mime_type' => $accepted_mimes,
            'numberposts'    => - 1,
            'post_status'    => null,
            'post_parent'    => $post_id,
            'order'          => 'ASC'
        );

        $attachments     = get_posts( $args );
        $attach_order    = get_post_meta( $post_id, 'attach_order', true );
        $attach_order = $attach_order ? $attach_order : array();
        $listed_attach   = array();
        $unlisted_attach = array();

        foreach ( $attachments as $attachment ) {
            if ( in_array( $attachment->ID, $attach_order ) ) :
                array_push($listed_attach, $attachment);
            else:
                array_push($unlisted_attach, $attachment);
            endif;
        }

        usort( $listed_attach, function ( $a, $b ) use ( $attach_order ) {
            $order    = $attach_order;
            $flipped  = array_flip( $order );
            $leftPos  = $flipped[ $a->ID ];
            $rightPos = $flipped[ $b->ID ];

            return $leftPos >= $rightPos;
        } );

        $attachments = array_merge( $listed_attach, $unlisted_attach );

        return $attachments;
    }

    function update_media_attach_state_callback() {
        global $wpdb;

        $valid_ids_array = array();
        $media_ids_array = is_array($_POST['media_ids_array']) ? $_POST['media_ids_array'] : array($_POST['media_ids_array']);
        $parent_post_id = intval(esc_sql($_POST['the_parent_id']));
        $is_attach = esc_sql($_POST['attach']);
        $pp_for_query = $is_attach === 'true' ? $parent_post_id : 0; //0 for detach

        foreach ($media_ids_array as $id) {
            $id = intval(esc_sql($id));
            array_push($valid_ids_array,$id);
        }

        $ids_string = implode( ',', $valid_ids_array );
        $wpdb->query( $wpdb->prepare( "UPDATE $wpdb->posts SET post_parent = %d WHERE post_type = 'attachment' AND ID IN ( $ids_string )", $pp_for_query ) );
        $this->dd_attachment_custom_box();
        die();
    }
}

$dd_attachments = new dd_attachments();